# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def emulation_pcc_info(self, mode, handle, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    emulation_pcc_info
		
		 Description:
		    Retrieves information about the PCEP protocol.
		
		 Synopsis:
		    emulation_pcc_info
		x       -mode         CHOICES stats
		x                     CHOICES clear_stats
		x                     CHOICES learned_info
		        -handle       ANY
		        [-port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$]
		
		 Arguments:
		x   -mode
		    -handle
		        The PCEP PCE handle to act upon.
		    -port_handle
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		    key:status                            value:$::SUCCESS | $::FAILURE
		    If status is failure, detailed information provided.
		    key:log                               value:If status is failure, detailed information provided.
		    Sessions Up
		x   key:sessions_up                       value:Sessions Up
		    Sessions Down
		x   key:sessions_down                     value:Sessions Down
		    Sessions Not Started
		x   key:sessions_not_started              value:Sessions Not Started
		    Sessions Total
		x   key:sessions_total                    value:Sessions Total
		    Session Flap Count
		x   key:session_flap_count                value:Session Flap Count
		    Open Message Rx
		x   key:open_message_rx                   value:Open Message Rx
		    Open Message Tx
		x   key:open_message_tx                   value:Open Message Tx
		    Keepalive Message Rx
		x   key:keepalive_message_rx              value:Keepalive Message Rx
		    Keepalive Message Tx
		x   key:keepalive_message_tx              value:Keepalive Message Tx
		    Close Message Rx
		x   key:close_message_rx                  value:Close Message Rx
		    Close Message Tx
		x   key:close_message_tx                  value:Close Message Tx
		    Unknown Message Rx
		x   key:unknown_message_rx                value:Unknown Message Rx
		    Total TCP Connection Established
		x   key:total_tcp_connection_established  value:Total TCP Connection Established
		    PCInitiate Message Rx
		x   key:pcinitiate_message_rx             value:PCInitiate Message Rx
		    Remove Initiated LSP Rx
		x   key:remove_initiated_lsp_rx           value:Remove Initiated LSP Rx
		    PCRpt Message Tx
		x   key:pcrpt_message_tx                  value:PCRpt Message Tx
		    PCUpdate Message Rx
		x   key:pcupdate_message_rx               value:PCUpdate Message Rx
		    PCErr Message Rx
		x   key:pcerr_message_rx                  value:PCErr Message Rx
		    PCErr Message Tx
		x   key:pcerr_message_tx                  value:PCErr Message Tx
		    Total Initiated LSP Rx
		x   key:total_initiated_lsp_rx            value:Total Initiated LSP Rx
		    Sync LSP Tx
		x   key:sync_lsp_tx                       value:Sync LSP Tx
		    PCReq Message Tx
		x   key:pcreq_message_tx                  value:PCReq Message Tx
		    PCRep Message Rx
		x   key:pcrep_message_rx                  value:PCRep Message Rx
		    Total Requested LSP Tx
		x   key:total_requested_lsp_tx            value:Total Requested LSP Tx
		    Total Responded LSP Rx
		x   key:total_responded_lsp_rx            value:Total Responded LSP Rx
		    No Path Rx
		x   key:no_path_rx                        value:No Path Rx
		    RSVP Going Up LSP Tx
		x   key:rsvp_going_up_lsp_tx              value:RSVP Going Up LSP Tx
		    RSVP Up LSP Tx
		x   key:rsvp_up_lsp_tx                    value:RSVP Up LSP Tx
		    RSVP Remove LSP Tx
		x   key:rsvp_remove_lsp_tx                value:RSVP Remove LSP Tx
		    RSVP Delegated LSP Tx
		x   key:rsvp_delegated_lsp_tx             value:RSVP Delegated LSP Tx
		    RSVP Revoked LSP Tx
		x   key:rsvp_revoked_lsp_tx               value:RSVP Revoked LSP Tx
		    RSVP Delegation Returned LSP Rx
		x   key:rsvp_delegation_returned_lsp_rx   value:RSVP Delegation Returned LSP Rx
		    SR Delegated LSP Tx
		x   key:sr_delegated_lsp_tx               value:SR Delegated LSP Tx
		    SR Revoked LSP Tx
		x   key:sr_revoked_lsp_tx                 value:SR Revoked LSP Tx
		    SR Delegation Returned LSP Rx
		x   key:sr_delegation_returned_lsp_rx     value:SR Delegation Returned LSP Rx
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		    Coded versus functional specification.
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'emulation_pcc_info', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
